@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.42.2.13 2005/06/05 15:34:49 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.64.4.3 2006/11/10 00:20:19 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -227,8 +227,8 @@ afs_DCWhichBucket(afs_int32 phase, afs_int32 bucket)
     if (!splitdcache) 
 	return 0;
 
-    afs_pct1 = afs_blocksUsed_1*100/afs_cacheBlocks;    
-    afs_pct2 = afs_blocksUsed_2*100/afs_cacheBlocks;
+    afs_pct1 = afs_blocksUsed_1 / (afs_cacheBlocks / 100);
+    afs_pct2 = afs_blocksUsed_2 / (afs_cacheBlocks / 100);
 
     /* Short cut: if we don't know about it, try to kill it */
     if (phase < 2 && afs_blocksUsed_0) 
@@ -346,16 +346,14 @@ afs_CacheTruncateDaemon(void)
     u_int counter;
     u_int cb_lowat;
     u_int dc_hiwat =
-	(100 - CM_DCACHECOUNTFREEPCT +
-	 CM_DCACHEEXTRAPCT) * afs_cacheFiles / 100;
+	PERCENT((100 - CM_DCACHECOUNTFREEPCT + CM_DCACHEEXTRAPCT), afs_cacheFiles);
     afs_min_cache =
 	(((10 * AFS_CHUNKSIZE(0)) + afs_fsfragsize) & ~afs_fsfragsize) >> 10;
 
     osi_GetuTime(&CTD_stats.CTD_afterSleep);
     afs_TruncateDaemonRunning = 1;
     while (1) {
-	cb_lowat = ((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT)
-		    * afs_cacheBlocks) / 100;
+	cb_lowat = PERCENT((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT), afs_cacheBlocks);
 	MObtainWriteLock(&afs_xdcache, 266);
 	if (afs_CacheTooFull) {
 	    int space_needed, slots_needed;
@@ -722,7 +720,7 @@ afs_GetDownD(int anumber, int *aneedSpace, afs_int32 buckethint)
 			MReleaseWriteLock(&afs_xdcache);
 		    }
 
-		    AFS_FAST_RELE(tvc);
+		    afs_PutVCache(tvc); /*XXX was AFS_FAST_RELE?*/
 		    MObtainWriteLock(&afs_xdcache, 528);
 		    if (afs_indexFlags[tdc->index] &
 			(IFDataMod | IFDirtyPages | IFAnyPages))
@@ -916,7 +914,7 @@ afs_FlushDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if (afs_blocksUsed <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -949,7 +947,7 @@ afs_FreeDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -999,7 +997,7 @@ afs_DiscardDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -1084,7 +1082,7 @@ afs_MaybeFreeDiscardedDCache(void)
 
     while (afs_blocksDiscarded
 	   && (afs_blocksUsed >
-	       (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+	       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	afs_FreeDiscardedDCache();
     }
     return 0;
@@ -1966,7 +1964,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	       ICL_HANDLE_OFFSET(Position));
     if ((aflags & 4) && (hiszero(avc->m.DataVersion)))
 	doAdjustSize = 1;
-    if ((aflags & 4) && (abyte == Position) && (tlen >= size))
+    if ((AFS_CHUNKTOBASE(chunk) >= avc->m.Length) ||
+	 ((aflags & 4) && (abyte == Position) && (tlen >= size)))
 	overWriteWholeChunk = 1;
     if (doAdjustSize || overWriteWholeChunk) {
 #if	defined(AFS_AIX32_ENV) || defined(AFS_SGI_ENV)
@@ -2117,7 +2116,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	/* Sleep here when cache needs to be drained. */
 	if (setLocks && !slowPass
 	    && (afs_blocksUsed >
-		(CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+		PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	    /* Make sure truncate daemon is running */
 	    afs_MaybeWakeupTruncateDaemon();
 	    ObtainWriteLock(&tdc->tlock, 614);
@@ -2126,7 +2125,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    ReleaseWriteLock(&tdc->lock);
 	    ReleaseReadLock(&avc->lock);
 	    while ((afs_blocksUsed - afs_blocksDiscarded) >
-		   (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 		afs_WaitForCacheDrain = 1;
 		afs_osi_Sleep(&afs_WaitForCacheDrain);
 	    }
@@ -2222,6 +2221,50 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    code = afs_CFileWrite(file, 0, dynrootDir, size);
 	    afs_PutDynroot();
 
+	    if (code == size)
+		code = 0;
+	    else
+		code = -1;
+
+	    tdc->validPos = Position + size;
+	    afs_CFileTruncate(file, size);	/* prune it */
+        } else if (afs_IsDynrootMount(avc)) {
+	    char *dynrootDir;
+	    int dynrootLen;
+
+	    afs_GetDynrootMount(&dynrootDir, &dynrootLen, &tsmall->OutStatus);
+
+	    dynrootDir += Position;
+	    dynrootLen -= Position;
+	    if (size > dynrootLen)
+		size = dynrootLen;
+	    if (size < 0)
+		size = 0;
+	    code = afs_CFileWrite(file, 0, dynrootDir, size);
+	    afs_PutDynroot();
+
+	    if (code == size)
+		code = 0;
+	    else
+		code = -1;
+
+	    tdc->validPos = Position + size;
+	    afs_CFileTruncate(file, size);	/* prune it */
+        } else if (afs_IsDynrootMount(avc)) {
+	    char *dynrootDir;
+	    int dynrootLen;
+
+	    afs_GetDynrootMount(&dynrootDir, &dynrootLen, &tsmall->OutStatus);
+
+	    dynrootDir += Position;
+	    dynrootLen -= Position;
+	    if (size > dynrootLen)
+		size = dynrootLen;
+	    if (size < 0)
+		size = 0;
+	    code = afs_CFileWrite(file, 0, dynrootDir, size);
+	    afs_PutDynroot();
+
 	    if (code == size)
 		code = 0;
 	    else
@@ -2800,7 +2843,7 @@ afs_MemGetDSlot(register afs_int32 aslot, register struct dcache *tmpdc)
     if (CheckLock(&afs_xdcache) != -1)
 	osi_Panic("getdslot nolock");
     if (aslot < 0 || aslot >= afs_cacheFiles)
-	osi_Panic("getdslot slot");
+	osi_Panic("getdslot slot %d (of %d)", aslot, afs_cacheFiles);
     tdc = afs_indexTable[aslot];
     if (tdc) {
 	QRemove(&tdc->lruq);	/* move to queue head */
@@ -2892,7 +2935,7 @@ afs_UFSGetDSlot(register afs_int32 aslot, register struct dcache *tmpdc)
     if (CheckLock(&afs_xdcache) != -1)
 	osi_Panic("getdslot nolock");
     if (aslot < 0 || aslot >= afs_cacheFiles)
-	osi_Panic("getdslot slot");
+	osi_Panic("getdslot slot %d (of %d)", aslot, afs_cacheFiles);
     tdc = afs_indexTable[aslot];
     if (tdc) {
 	QRemove(&tdc->lruq);	/* move to queue head */
