@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/libadmin/client/afs_clientAdmin.c,v 1.11 2005/07/21 04:56:02 shadow Exp $");
 
 #include <afs/stds.h>
 #include "afs_clientAdmin.h"
@@ -191,8 +193,7 @@ afsclient_TokenGetExisting(const char *cellName, void **tokenHandle,
 	 * The token has been retrieved successfully, initialize
 	 * the rest of the token handle structure
 	 */
-	strncpy(t_handle->cell, cellName, MAXCELLCHARS);
-        t_handle->cell[MAXCELLCHARS - 1] = '\0';
+	strcpy(t_handle->cell, cellName);
 	t_handle->afs_token_set = 1;
 	t_handle->from_kernel = 1;
 	t_handle->kas_token_set = 0;
@@ -567,8 +568,7 @@ afsclient_TokenGetNew(const char *cellName, const char *principal,
 
 	if ((GetAFSToken(cellName, principal, password, t_handle, &tst))
 	    && (GetKASToken(cellName, principal, password, t_handle, &tst))) {
-	    strncpy(t_handle->cell, cellName, MAXCELLCHARS);
-            t_handle->cell[MAXCELLCHARS - 1] = '\0';
+	    strcpy(t_handle->cell, cellName);
 	    t_handle->from_kernel = 0;
 	    t_handle->afs_token_set = 1;
 	    t_handle->kas_token_set = 1;
@@ -868,8 +868,7 @@ afsclient_CellOpen(const char *cellName, const void *tokenHandle,
      * information for each server in the cell
      */
 
-    strncpy(c_handle->working_cell, cellName, MAXCELLCHARS);
-    c_handle->working_cell[MAXCELLCHARS - 1] = '\0';
+    strcpy(c_handle->working_cell, cellName);
     if (!(tdir = afsconf_Open(AFSDIR_CLIENT_ETC_DIRPATH))) {
 	tst = ADMCLIENTBADCLIENTCONFIG;
 	goto fail_afsclient_CellOpen;
@@ -879,8 +878,7 @@ afsclient_CellOpen(const char *cellName, const void *tokenHandle,
      * We must copy the cellName here because afsconf_GetCellInfo
      * actually writes over the cell name it is passed.
      */
-    strncpy(copyCell, cellName, MAXCELLCHARS);
-    copyCell[MAXCELLCHARS - 1] ='\0';
+    strncpy(copyCell, cellName, MAXCELLCHARS - 1);
     for (i = 0; (i < NUM_SERVER_TYPES); i++) {
 	if (i == KAS) {
 	    tst =
@@ -1530,7 +1528,7 @@ afsclient_ACLEntryAdd(const char *directory, const char *user,
      */
 
     is_dfs =
-	sscanf(old_acl_string, "%d dfs:%d %1024s", &cur_acl.nplus, &cur_acl.dfs,
+	sscanf(old_acl_string, "%d dfs:%d %s", &cur_acl.nplus, &cur_acl.dfs,
 	       cur_acl.cell);
     ptr = strchr(old_acl_string, '\n');
     ptr++;
@@ -1555,7 +1553,7 @@ afsclient_ACLEntryAdd(const char *directory, const char *user,
      */
 
     for (i = 0; i < (cur_acl.nplus + cur_acl.nminus); i++) {
-	sscanf(ptr, "%63s%d\n", cur_user, &cur_user_acl);
+	sscanf(ptr, "%s%d\n", cur_user, &cur_user_acl);
 	/*
 	 * Skip the entry for the user we are replacing/adding
 	 */
@@ -1965,7 +1963,6 @@ afsclient_AFSServerGetBegin(const void *cellHandle, void **iterationIdP,
 	if (host != NULL) {
 	    strncpy(serv->server[iserv].serverName, host->h_name,
 		    AFS_MAX_SERVER_NAME_LEN);
-            serv->server[iserv].serverName[AFS_MAX_SERVER_NAME_LEN - 1] = '\0';
 	}
     }
     UNLOCK_GLOBAL_MUTEX;
